package org.kldp.jmassmailer.mail;

import java.io.UnsupportedEncodingException;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import org.kldp.jmassmailer.data.*;
import org.kldp.jmassmailer.ui.*;
import com.sun.mail.smtp.*;

/*
 * Created on 2005. 5. 2
 */

/**
 * @author (JeongSoo, Park)
 */
public class SessionHandler implements SessionModelInterface, Runnable {
    private Session session;	 //TODO In the future, count a number of mail service provider from list, and make sessions per each provider.
    private PeopleList people;
    private String encoding;
    private String contentType;
    private String envelopeFrom;
    private String subject;
    private String text;
    
    // Message header and body
    private ArrayList messages; 
    private Address[] replyto;
    private Address[] from;
    
    // Observers
    private ArrayList transferObservers = new ArrayList();
    
    /**
     * Constructor
     */
    public SessionHandler() {
//        contentType = "text/html; charset=\"" + encoding +"\"";
    	encoding = "ks_c_5601-1987";
    	contentType = "text/html";
    }
    
	/**
	 * Constructor
	 * @param properties Properties of session
	 * @param auth Authenticator needs on authentication
	 */
    public SessionHandler(Properties properties, Authenticator auth) {
        this();
        createSession(properties, auth);        
    }
      
    
    /**
     * When dummy constructor is called, create session explicity
     * 
     * @param properties
     * @param auth
     */
    public void createSession(Properties properties, Authenticator auth) {
        session = Session.getInstance(properties, auth);
    }
    
    
    /**
     * Set people list   
     * 
     * @param people
     */
    public void setPeopleList(PeopleList people) {
        this.people = people;
    }
    
    
    /**
     * @param replyto Reply-to address
     * @throws AddressException Throws exception when reply-to address is invalid
     */
    public void setReplyTo(String replyto) throws AddressException {
    	this.replyto = new Address[1]; 
    	try {
			this.replyto[0] = new InternetAddress(replyto);
		} catch (AddressException e) {
			throw new AddressException("Reply-to address format is invalid");
		}
    }

    public void setFrom(String email) throws AddressException {
    	this.from = new Address[1];
    	this.from[0] = new InternetAddress(email);
    }
    
    public void setFrom(String email, String name) throws AddressException {
    	this.from = new Address[1]; 
    	String emailWithName = null;
    	
    	try {
			emailWithName = MimeUtility.encodeText(name, encoding, "B") + " <" + email + ">";
		} catch (UnsupportedEncodingException e1) {
			emailWithName = email;
		}
		
		this.from[0] = new InternetAddress(emailWithName);
    }

    public void setMessage(String subject, String text){
    	this.subject = subject;
    	this.text = text;
    }

    public void setContentType(String contentType){
    	this.contentType = contentType;
    }
    
    public void setEncoding(String encoding){
    	this.encoding = encoding;
    }
    

    
    /* make messages through each person 
     * with personlized text and e-mail address.
     */
    public void generateMessages() throws MessagingException {
    	messages = new ArrayList();
    	
        if(people == null)
            throw new NullPointerException("PeopleList isn't allocated");
        
        Iterator i = people.iterator();
        
        while(i.hasNext()){            
            SMTPMessage message = new SMTPMessage(session);
            Person person = (Person)i.next();
            String personalizedContent = Person.personalizeText(text, person);
            String personalizedSubject = Person.personalizeText(subject, person);
            String emailWithName;
			try {
				emailWithName = MimeUtility.encodeText(person.getName(), encoding, "B") + " <" + person.getEmail() + ">";
			} catch (UnsupportedEncodingException e1) {
				emailWithName = person.getEmail();
			}
			
            try {            	
                message.addRecipient(Message.RecipientType.TO, new InternetAddress(emailWithName));
                message.addFrom(from);
                message.setEnvelopeFrom(envelopeFrom);
                if(replyto != null) message.setReplyTo(replyto);
            } catch(MessagingException e){
                throw new MessagingException("E-mail address is invalid : " + person.getEmail() + " (" + person.getName() + ")");
            }
                
            try {                
                message.setSubject(personalizedSubject, encoding);
                message.setContent(personalizedContent, contentType + "; charset=\"" + encoding + "\"");
            } catch (MessagingException e) {
//                throw new MessagingException("SessionHandler : The charset encoding process fails during setSubject() or setContent()");
            	;
            }
            
            messages.add(message);
        }                
    }
    
    /**
     * Transfer messages through a session.
     * 
     * @throws MessagingException
     */
    public void transfer() throws NoSuchProviderException, AuthenticationFailedException, MessagingException {
    	generateMessages();
    	
    	int maximum = messages.size();
    	int counter = 0;
    	int percent = 0;
    	
    	Iterator i = messages.iterator();
    	Message message = null;
    	Address[] recipients = null;
    	
    	while(i.hasNext()){
    		message = (Message)i.next();
    		recipients = message.getAllRecipients();
    		
    		try {
    			Transport.send(message);
    		} catch (MessagingException e2) {
    			throw new MessagingException("Message sending error during transfer message: " + e2.getMessage());
    		}
    		
    		percent = (counter / maximum) * 100;
    		InternetAddress email = (InternetAddress)recipients[0];
    		notifyMessageSend(percent, email.toString());
    		counter++;
    	}
    	
    	notifyMessageSendFinished(); //TODO pass a list which is failed to transfer
    }
    
    public void registerObserver(TransferObserver o){
    	transferObservers.add(o);
    }
    
    public void removeObserver(TransferObserver o){
    	int i = transferObservers.indexOf(o);
		if (i >= 0) {
			transferObservers.remove(i);
		}
    }
    
    public void notifyMessageSend(int percent, String email){
    	for(int i = 0; i < transferObservers.size(); i++){
    		TransferObserver o = (TransferObserver)transferObservers.get(i);
    		o.updateMessageSend(percent, email);
    	}
    }
    
    public void notifyMessageSendFinished(){
    	for(int i = 0; i < transferObservers.size(); i++){
    		TransferObserver o = (TransferObserver)transferObservers.get(i);
    		o.updateMessageSendFinished();
    	}    	
    }
    
    public void run(){
    	try {
			transfer();
		} catch (NoSuchProviderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AuthenticationFailedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MessagingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
}